//+------------------------------------------------------------------+
//|                                             SaveHistoryToHST.mq5 |
//|                                                        avoitenko |
//|                        https://login.mql5.com/en/users/avoitenko |
//+------------------------------------------------------------------+
#property copyright "avoitenko"
#property link      "https://login.mql5.com/en/users/avoitenko"
#property version   "2.00"
#property script_show_inputs

//--- defines
#define COPYRIGHT "(C)opyright 2003, MetaQuotes Software Corp."
#define OLD_RECORD_SIZE 44
//---
struct HistoryHeader
  {
   uint              version;
   char              copyright[64];
   char              symbol[12];
   uint              period;
   uint              digits;
   uint              time_sign;
   uint              last_sync;
   uint              unused[13];
  };
//---
enum ENUM_HISTORY_FORMAT
  {
   HISTORY_NEW,//New History Format
   HISTORY_OLD//Old History Format
  };
//--- input variables
input ENUM_HISTORY_FORMAT InpHistoryFormat=HISTORY_NEW;//Format type
input uint InpDataCount=5000;//Number of bars
input string InpSymbol="";//Symbol
input ushort InpTimeframe=0;//Timeframe, minutes
//--- global variables
MqlRates rates[];
HistoryHeader header;
//+------------------------------------------------------------------+
//|   OnStart                                                        |
//+------------------------------------------------------------------+
void OnStart()
  {
//--- check max history data
   int bars=Bars(_Symbol,_Period);
   int count=(int)fmin(InpDataCount,bars);
   if(InpDataCount==0)count=bars;

//--- get data
   ResetLastError();
   if(CopyRates(_Symbol,_Period,0,count,rates)!=count)
     {
      PrintFormat("Error CopyRates, error: %d",GetLastError());
      return;
     }

//--- symbol
   string symbol=StringSubstr(_Symbol,0,11);
   if(InpSymbol!="")
      symbol=StringSubstr(InpSymbol,0,11);

//--- timeframe
   uint period=PeriodSeconds(_Period)/60;
   if(InpTimeframe>0)
     {
      period=InpTimeframe;
      if(period>0x7FFF)period=0x7FFF;
     }

//--- file name      
   string file_name=StringFormat("%s%d.hst",symbol,period);

//--- header
   header.period = period;
   header.digits = _Digits;
   header.version=401;
   if(InpHistoryFormat==HISTORY_OLD) header.version=400;
   StringToCharArray(COPYRIGHT,header.copyright);
   StringToCharArray(symbol,header.symbol);
   header.time_sign=(int)TimeCurrent();

//--- open file
   ResetLastError();
   int F=FileOpen(file_name,FILE_BIN|FILE_ANSI|FILE_WRITE);
   if(F==INVALID_HANDLE)
     {
      PrintFormat("Error FileOpen operation, error: %d",GetLastError());
      return;
     }

//--- write header
   FileWriteStruct(F,header);

//--- write data
   int percent;
   int delta= count/10;
   for(int i=0; i<count && !_StopFlag; i++)
     {
      if(InpHistoryFormat==HISTORY_OLD)
        {
         FileWriteInteger(F,(int)rates[i].time,INT_VALUE);
         FileWriteDouble(F, rates[i].open);
         FileWriteDouble(F, rates[i].low);
         FileWriteDouble(F, rates[i].high);
         FileWriteDouble(F, rates[i].close);
         FileWriteDouble(F, rates[i].tick_volume);
        }
      if(InpHistoryFormat==HISTORY_NEW)
         FileWriteStruct(F,rates[i]);

      //--- show percent
      if(fmod(i,delta)==0)
        {
         percent=(i/delta)*10;
         Comment("\nSaved "+IntegerToString(percent)+"% of data");
         Sleep(100);
        }
     }
//--- close file
   FileClose(F);

//--- print path
   Print("Path: ",TerminalInfoString(TERMINAL_DATA_PATH),"\\MQL5\\Files\\",file_name);
   Comment("");
  }
//+------------------------------------------------------------------+
